/// <reference types="node" />
import { Readable } from 'stream';
import { Writable } from 'stream';
import { EventEmitter } from 'events';
import { IncomingHttpHeaders } from 'http';
import { StringDecoder } from 'string_decoder';
import createError = require('http-errors');
import { FormOptions, Fn, ObjectAny, HoldEmitQueueItem, PassThroughExt, OpenedFile, NodeReq, PartEvent, FormFile } from './types';
export declare class Form extends Writable {
    /**
     * The amount of bytes received for this form so far.
     */
    bytesReceived: number;
    /**
     * The expected number of bytes in this form.
     */
    bytesExpected: number;
    protected error: Error;
    protected autoFields: boolean;
    protected autoFiles: boolean;
    protected maxFields: number;
    protected maxFieldsSize: number;
    protected maxFilesSize: number;
    protected uploadDir: string;
    protected encoding: BufferEncoding;
    protected openedFiles: OpenedFile[];
    protected totalFieldSize: number;
    protected totalFieldCount: number;
    protected totalFileSize: number;
    protected flushing: number;
    protected backpressure: boolean;
    protected writeCbs: Fn[];
    protected emitQueue: HoldEmitQueueItem[];
    protected destStream: PassThroughExt;
    protected req: NodeReq;
    protected waitend: boolean;
    protected boundOnReqAborted: Fn;
    protected boundOnReqEnd: Fn;
    protected partTransferEncoding: string;
    protected partHeaders: IncomingHttpHeaders;
    protected headerFieldDecoder: StringDecoder;
    protected headerValueDecoder: StringDecoder;
    protected headerField: string;
    protected headerValue: string;
    protected partFilename: string;
    protected partName: string;
    protected boundary: Buffer;
    protected index: number;
    protected partDataMark: number;
    protected headerValueMark: number;
    protected headerFieldMark: number;
    protected partBoundaryFlag: boolean;
    protected state: number;
    protected lookbehind: Buffer;
    protected boundaryChars: {
        [x: string]: boolean;
    };
    constructor(options?: FormOptions);
    parse(req: NodeReq, cb?: (err?: Error & {
        statusCode?: number;
    }, fields?: ObjectAny, files?: ObjectAny) => void): void;
    /**
     * Unless you supply a callback to `form.parse`, you definitely want to handle
     * this event. Otherwise your server *will* crash when users submit bogus
     * multipart requests!
     *
     * Only one `error` event can ever be emitted, and if an `error` event is
     * emitted, then `close` will not be emitted.
     *
     * If the error would correspond to a certain HTTP response code, the `err` object
     * will have a `statusCode` property with the value of the suggested HTTP response
     * code to send back.
     *
     * Note that an `error` event will be emitted both from the `form` and from the
     * current `part`.
     */
    on(event: 'error', listener: (err: Error & {
        statusCode?: number;
    }) => void): this;
    /**
     * Emitted when a part is encountered in the request.
     * Parts for fields are not emitted when `autoFields` is on, and likewise parts
     * for files are not emitted when `autoFiles` is on.
     *
     * `part` emits 'error' events! Make sure you handle them.
     *
     * You *must* act on the part by reading it.
     * If you want to ignore it, just call `part.resume()`.
     */
    on(event: 'part', listener: (part: PartEvent) => void): this;
    /**
     * Emitted when the request is aborted. This event will be followed shortly
     * by an `error` event. In practice you do not need to handle this event.
     */
    on(event: 'aborted', listener: () => void): this;
    /**
     * Emitted when a chunk of data is received for the form. The `bytesReceived`
     * argument contains the total count of bytes received for this form so far. The
     * `bytesExpected` argument contains the total expected bytes if known, otherwise
     * `null`.
     */
    on(event: 'progress', listener: (bytesReceived?: number, bytesExpected?: number) => void): this;
    /**
     * Emitted after all parts have been parsed and emitted. Not emitted if an `error`
     * event is emitted.
     *
     * If you have `autoFiles` on, this is not fired until all the data has been
     * flushed to disk and the file handles have been closed.
     *
     * This is typically when you would send your response.
     */
    on(event: 'close', listener: () => void): this;
    /**
     * **By default multiparty will not touch your hard drive.** But if you add this
     * listener, multiparty automatically sets `form.autoFiles` to `true` and will
     * stream uploads to disk for you.
     *
     * **The max bytes accepted per request can be specified with `maxFilesSize`.**
     */
    on(event: 'file', listener: (name?: string, file?: FormFile) => void): this;
    on(event: 'field', listener: (name?: string, value?: string) => void): this;
    on(event: 'drain', listener: () => void): this;
    on(event: 'finish', listener: () => void): this;
    on(event: 'resume', listener: () => void): this;
    on(event: 'pipe', listener: (src: Readable) => void): this;
    on(event: 'unpipe', listener: (src: Readable) => void): this;
    on(event: 'newListener', listener: (event: string) => void): this;
    _write(buffer: Buffer, encoding: string, cb: Fn): void;
    onParsePartBegin(): void;
    onParseHeaderField(b: Buffer): void;
    onParseHeaderValue(b: Buffer): void;
    onParseHeaderEnd(): void;
    onParsePartData(b: Buffer): void;
    onParsePartEnd(): void;
    onParseHeadersEnd(offset: number): createError.HttpError;
    protected onReqEnd(): void;
    protected handleError(err: Error): void;
    protected onReqAborted(): void;
    protected clearPartVars(): void;
    protected parseFilename(headerValue: string): string;
    protected lower(c: number): number;
    protected flushWriteCbs(): void;
    protected getBytesExpected(headers: IncomingHttpHeaders): number;
    protected beginFlush(): void;
    protected endFlush(): void;
    protected cleanupOpenFiles(): void;
    protected holdEmitQueue(eventEmitter?: EventEmitter): (cb: Fn) => void;
    protected errorEventQueue(eventEmitter: EventEmitter, err: Error): void;
    protected handlePart(partStream: PassThroughExt): void;
    protected handleFile(fileStream: PassThroughExt): void;
    protected handleField(fieldStream: PassThroughExt): void;
    protected setUpParser(boundary: string): void;
}
